/*
 * Bat handling functions
 *
 * Copyright (c) 2000 Michael Pearson <alcaron@ozemail.com.au>
 *
 * This file is licensed under the GNU General Public License. See the file
 * "COPYING" for more details.
 */

#include "breakout.h"
#include "bat.h"
#include "block.h"
#include "anim.h"
#include "gui.h"

#define LASER_SPEED 19
#define LASER_WIDTH 15
#define LASER_HEIGHT 15

/* Internal functions */
static void move_bat(Game *game);
static void change_to_laser(Bat *bat);
static void reset_from_laser(Bat *bat);
static void iterate_bat_laser(Game *game);

static void move_bat(Game *game) {
        Bat *bat;
        gint bat_move;

        /* Included for brevity */
        bat = game->bat;

        if(game->flags.keyboard_control) {
                bat_move = game->keyboard_move;
                if(bat->geometry.x1 + bat_move > 0
                                && bat->geometry.x2 + bat_move < GAME_WIDTH) {
                        bat->geometry.x1 += bat_move;
                        bat->geometry.x2 += bat_move;
                } else {
                        if(bat->geometry.x2 + bat_move >= GAME_WIDTH) {
                                bat->geometry.x2 = GAME_WIDTH;
                                bat->geometry.x1 = GAME_WIDTH - BAT_WIDTH;
                        } else {
                                bat->geometry.x1 = 0;
                                bat->geometry.x2 = BAT_WIDTH;
                        }
                }
        } else if(game->flags.mouse_control) {
                bat_move = game->mouse_move;
                if(bat_move < BAT_WIDTH / 2)
                        bat_move = BAT_WIDTH /2;
                else if(bat_move > GAME_WIDTH - BAT_WIDTH /2)
                        bat_move = GAME_WIDTH - BAT_WIDTH /2;

                bat->geometry.x1 = bat_move - BAT_WIDTH / 2;
                bat->geometry.x2 = bat->geometry.x1 + BAT_WIDTH;
        }
        update_canvas_position((Entity *) bat);
}

/* Creates a new bat at the start of a game */
void new_bat(Game *game) {
	Bat *bat;

        bat = g_malloc(sizeof(Bat));
        g_assert(bat);

        bat->geometry.y2 = GAME_HEIGHT - BLOCK_WALL_PADDING;
        bat->geometry.y1 = bat->geometry.y2 - BAT_HEIGHT;
        bat->geometry.x1 = GAME_WIDTH / 2 - BAT_WIDTH / 2;
        bat->geometry.x2 = bat->geometry.x1 + BAT_WIDTH;
        g_assert(bat->geometry.x1 > BLOCK_WALL_PADDING);
        g_assert(bat->geometry.x2 < GAME_WIDTH - BLOCK_WALL_PADDING);

        bat->animation = get_static_animation(ANIM_BAT_DEFAULT);
	bat->child = NULL;
        add_to_canvas((Entity *) bat);
        bat->type = BAT_DEFAULT;

	game->bat = bat;
}

/* Iterates the bat */
void iterate_bat(Game *game) {
	switch(game->bat->type) {
		case BAT_DEFAULT :
			move_bat(game);
			break;
		case BAT_LASER :
			move_bat(game);
			iterate_bat_laser(game);
			break;
		default :
			g_assert_not_reached();
	}
}

/* Changes the bat type. Don't use this to change a bat back to DEFAULT, use
 * reset_bat_type instead */
void change_bat_type(Game *game, BatType type) {
	if(game->bat->type != BAT_DEFAULT)
		reset_bat_type(game);

	switch(type) {
		case BAT_DEFAULT :
			g_warning("DEVELOPER: Don't use this! Use reset_bat_type instead!");
			reset_bat_type(game);
			break;
		case BAT_LASER :
			change_to_laser(game->bat);
			break;
		default :
			g_assert_not_reached();
	}
}

/* Resets the bat back to BAT_DEFAULT. Handles the destruction of anything
 * that a non-standard bat type may spawn */
void reset_bat_type(Game *game) {
	switch(game->bat->type) {
		case BAT_DEFAULT :
			break;
		case BAT_LASER :
			reset_from_laser(game->bat);
			break;
		default :
			g_assert_not_reached();
	}
}

/* Destroys the bat, de-allocating its resources and suchlike */
void destroy_bat(Game *game) {
	reset_bat_type(game);
	remove_from_canvas((Entity *) game->bat);
	g_free(game->bat);
	game->bat = NULL;
}

static void reset_from_laser(Bat *bat) {
	if(bat->child) {
		remove_from_canvas(bat->child);
		g_free(bat->child);
		bat->child = NULL;
	}

	remove_from_canvas((Entity *) bat);
	bat->animation = get_animation(ANIM_BAT_DEFAULT);
	add_to_canvas((Entity *) bat);

	bat->type = BAT_DEFAULT;
}

static void change_to_laser(Bat *bat) {
	remove_from_canvas((Entity *) bat);
	bat->animation = get_animation(ANIM_BAT_LASER);
	add_to_canvas((Entity *) bat);

	bat->type = BAT_LASER;
}

static void iterate_bat_laser(Game *game) {
	gboolean kill_child = FALSE;
	gint bat_center;
	Block *block;

	if(game->bat->child) {
		game->bat->child->geometry.y1 -= LASER_SPEED;
		game->bat->child->geometry.y2 -= LASER_SPEED;

		if(game->bat->child->geometry.y1 < 0) {
			kill_child = TRUE;
		} else if((block = find_block_from_position(game,
					(Entity *) game->bat->child))) {
			if(block->type != BLOCK_DEAD) {
				hit_block(game, block);
				kill_child = TRUE;
			}
		}

		if(kill_child) {
			remove_from_canvas(game->bat->child);
			g_free(game->bat->child);
			game->bat->child = NULL;
		} else {
			update_canvas_position(game->bat->child);
		}
	} else {
		if(game->fire1_pressed) {
			bat_center = game->bat->geometry.x1 + BAT_WIDTH / 2;

			game->bat->child = g_malloc(sizeof(Entity));
			game->bat->child->geometry.x1 = 
					bat_center - LASER_WIDTH / 2;
			game->bat->child->geometry.x2 =
					game->bat->child->geometry.x1
					+ LASER_WIDTH;
			game->bat->child->geometry.y2 = game->bat->geometry.y1;
			game->bat->child->geometry.y1 =
					game->bat->child->geometry.y2
					- LASER_HEIGHT;

			game->bat->child->animation = get_animation(ANIM_LASER);

			add_to_canvas(game->bat->child);
		}
	}
}
