/*
 * The GNOME-driven breakout GUI. Yes, it's messy, but it's also my first
 * ever real GNOME app.
 *
 * Copyright (c) 2000 Michael Pearson <alcaron@ozemail.com.au>
 *
 * This file is licensed under the GNU General Public License. See the file
 * "COPYING" for more details.
 */

#include "breakout.h"
#include "gui.h"
#include "gui-callbacks.h"
#include "game.h"
#include "anim.h"

/* See gui.h for more info */
static GuiInfo *gui;

/* Internal functions */
static void init_canvas(void);
static void init_labels(void);
static void init_menus(void);

/* Initialise the interface. */
void gui_init(Game *game, int argc, char **argv) {

	/* Initialise the basic app */
	gui = g_malloc(sizeof(GuiInfo));
	gui->game = game;
	gui->app = (GnomeApp *) gnome_app_new(PACKAGE,
			_("GNOME Breakout"));
	gtk_window_set_policy(GTK_WINDOW (gui->app), FALSE, FALSE, TRUE);
	gtk_signal_connect(GTK_OBJECT (gui->app), "delete_event",
		GTK_SIGNAL_FUNC (cb_sig_exit_game), gui);

	/* Bind the keypresses */
	gtk_signal_connect(GTK_OBJECT (gui->app), "key_press_event",
		GTK_SIGNAL_FUNC (cb_keydown), gui);
	gtk_signal_connect(GTK_OBJECT (gui->app), "key_release_event",
		GTK_SIGNAL_FUNC (cb_keyup), gui);

	/* Initialise the canvas */
	gui->vbox = gtk_vbox_new(FALSE, 0);
	gnome_app_set_contents(GNOME_APP (gui->app), gui->vbox);
	init_canvas();

	/* Initialise the labels */
	gui->appbar = gnome_appbar_new(FALSE, TRUE, GNOME_PREFERENCES_NEVER);
	gnome_app_set_statusbar(gui->app, gui->appbar);
	init_labels();

	/* Initialise the menus */
	init_menus();

	gtk_widget_show_all(GTK_WIDGET (gui->app));
}
	
static void init_canvas(void) {
	GdkImlibImage *image;

	/* Push the imlib colormap and visual, and make the canvas */
	gtk_widget_push_visual(gdk_imlib_get_visual());
	gtk_widget_push_colormap(gdk_imlib_get_colormap());
	gui->canvas = (GnomeCanvas *) gnome_canvas_new();

	/* Set the canvas attributes */
	gnome_canvas_set_pixels_per_unit(gui->canvas, 1);
	gtk_widget_set_usize(GTK_WIDGET(gui->canvas), GAME_WIDTH, GAME_HEIGHT);
	gnome_canvas_set_scroll_region(gui->canvas, 0, 0, GAME_WIDTH,
			GAME_HEIGHT);

	/* Make the canvas background black */
	gui->background = gnome_canvas_item_new(gnome_canvas_root(gui->canvas),
			GNOME_TYPE_CANVAS_RECT, "x1", 0.0, "x2",
			(double) GAME_WIDTH, "y1", 0.0, "y2",
			(double) GAME_HEIGHT, "fill_color", "black", NULL);
	gnome_canvas_item_hide(gui->background);

	/* Add the title image */
	image = gdk_imlib_load_image(PIXMAPDIR "/title.png");
	if(!image)
		g_error("Cannot find title image " PIXMAPDIR "/title.png");
	gdk_imlib_render(image, image->rgb_width, image->rgb_height);
	gui->title_image = gnome_canvas_item_new(
			gnome_canvas_root(gui->canvas),
			GNOME_TYPE_CANVAS_IMAGE, "image", image, 
			"x", 0.0, "y", 0.0,
			"width", (double) GAME_WIDTH, 
			"height", (double) GAME_HEIGHT,
			"anchor", GTK_ANCHOR_NORTH_WEST,
			NULL);

	gnome_canvas_update_now(gui->canvas);

	/* Pause on focus loss */
	gtk_signal_connect(GTK_OBJECT(gui->canvas), "enter-notify-event",
			GTK_SIGNAL_FUNC(cb_focus_in), gui);
	gtk_signal_connect(GTK_OBJECT(gui->canvas), "leave-notify-event",
			GTK_SIGNAL_FUNC(cb_focus_out), gui);

	/* Bind the mouse fire events */
	gtk_signal_connect(GTK_OBJECT(gui->canvas), "button_press_event",
			GTK_SIGNAL_FUNC(cb_canvas_button_press), gui);

	gtk_box_pack_start(GTK_BOX(gui->vbox), GTK_WIDGET(gui->canvas),
			FALSE, FALSE, 0);
}

static void init_labels(void) {
	GtkWidget *vseparator1;
	GtkWidget *vseparator2;

	gui->levellabel = gtk_label_new(_("Level: 0"));
	gtk_label_set_justify(GTK_LABEL(gui->levellabel), GTK_JUSTIFY_LEFT);
	gui->scorelabel = gtk_label_new(_("Score: 0"));
	gtk_label_set_justify(GTK_LABEL(gui->scorelabel), GTK_JUSTIFY_LEFT);
	gui->liveslabel = gtk_label_new(_("Lives: 0"));
	gtk_label_set_justify(GTK_LABEL(gui->liveslabel), GTK_JUSTIFY_LEFT);

	vseparator1 = gtk_vseparator_new();
	vseparator2 = gtk_vseparator_new();

	gtk_widget_set_sensitive(gui->scorelabel, FALSE);
	gtk_widget_set_sensitive(gui->liveslabel, FALSE);
	gtk_widget_set_sensitive(gui->levellabel, FALSE);

	gtk_box_pack_end(GTK_BOX(gui->appbar), gui->levellabel, FALSE, FALSE,
			5);
	gtk_box_pack_end(GTK_BOX(gui->appbar), vseparator2, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(gui->appbar), gui->scorelabel,
			FALSE, FALSE, 5);
	gtk_box_pack_end(GTK_BOX(gui->appbar), vseparator1, FALSE, FALSE, 0);
	gtk_box_pack_end(GTK_BOX(gui->appbar), gui->liveslabel,
			FALSE, FALSE, 5);
}

/* Adds an entity to the gnome canvas */
void add_to_canvas(Entity *entity) {
	entity->animation.canvas_item = gnome_canvas_item_new(
			gnome_canvas_root(GNOME_CANVAS(gui->canvas)),
			GNOME_TYPE_CANVAS_IMAGE,
			"image", entity->animation.pixmaps[entity->animation.frame_no],
			"x", (double) entity->geometry.x1,
			"y", (double) entity->geometry.y1,
			"width", (double) entity->geometry.x2 - entity->geometry.x1,
			"height", (double) entity->geometry.y2 - entity->geometry.y1,
			"anchor", GTK_ANCHOR_NORTH_WEST,
			NULL);
}

/* Remove an entity from the gnome canvas. Does not assume that the entity
 * actually has a canvas_item */
void remove_from_canvas(Entity *entity) {
	if(entity->animation.canvas_item) {
		gtk_object_destroy(GTK_OBJECT(entity->animation.canvas_item));
		entity->animation.canvas_item = NULL;
	}
}

/* Process all pending gnome events */
void process_gnome_events(void) {
	while(gtk_events_pending())
		gtk_main_iteration();
}

void update_gnome(Game *game) {
	char *score, *lives, *level;
	static gint32 oldscore = 0;
	static gint oldlives = 0, oldlevel = 0;

	if(oldlevel != game->level_no) {
		oldlevel = game->level_no;
		level = g_strdup_printf(_("Level: %d"), game->level_no);
		gtk_label_set_text(GTK_LABEL(gui->levellabel), level);
		g_free(level);
	}

	if(oldscore != game->score) {
		oldscore = game->score;
		score = g_strdup_printf(_("Score: %d"), game->score);
		gtk_label_set_text(GTK_LABEL(gui->scorelabel), score);
		g_free(score);
	}

	if(oldlives != game->lives) {
		oldlives = game->lives;
		lives = g_strdup_printf(_("Lives: %d"), game->lives);
		gtk_label_set_text(GTK_LABEL(gui->liveslabel), lives);
		g_free(lives);
	}

	gnome_canvas_update_now(gui->canvas);
	return;
}

/* Assumes that we already have an appbar */
static void init_menus() {
	GnomeUIInfo game_menu[] = {
		GNOMEUIINFO_MENU_NEW_GAME_ITEM(cb_new_game, gui),
		GNOMEUIINFO_MENU_PAUSE_GAME_ITEM(cb_pause_game, gui),
		GNOMEUIINFO_MENU_END_GAME_ITEM(cb_end_game, gui),
		GNOMEUIINFO_SEPARATOR,
		GNOMEUIINFO_MENU_PREFERENCES_ITEM(cb_preferences, gui),
		/* FIXME: Remove this when appropriate */
		GNOMEUIINFO_ITEM_DATA(_("_Kill ball"), 
				_("Kill the current ball if it gets stuck"),
				cb_kill_ball, gui, NULL),
		GNOMEUIINFO_MENU_SCORES_ITEM(cb_scores, gui),
		GNOMEUIINFO_SEPARATOR,
		GNOMEUIINFO_MENU_EXIT_ITEM(cb_exit_game, gui),
		GNOMEUIINFO_END
	};

	GnomeUIInfo help_menu[] = {
		GNOMEUIINFO_ITEM_DATA("_Help", "Help on this application",
				cb_help, gui, NULL),
		GNOMEUIINFO_MENU_ABOUT_ITEM(cb_show_about_box, gui),
		GNOMEUIINFO_END
	};

	GnomeUIInfo menubar[] = {
		GNOMEUIINFO_MENU_GAME_TREE(game_menu),
		GNOMEUIINFO_MENU_HELP_TREE(help_menu),
		GNOMEUIINFO_END
	};

	gnome_app_create_menus(gui->app, menubar);
	gnome_app_install_menu_hints(gui->app, menubar);
	gui->menu_new_game = game_menu[0].widget;
	gui->menu_pause = game_menu[1].widget;
	gui->menu_end_game = game_menu[2].widget;
	gtk_widget_set_sensitive(gui->menu_pause, FALSE);
	gtk_widget_set_sensitive(gui->menu_end_game, FALSE);
}

/* Updates the position of an item on the canvas. Assumes that the width or
 * height of the object hasn't changed */
void update_canvas_position(Entity *entity) {
	if(entity->animation.canvas_item) {
		gnome_canvas_item_set(GNOME_CANVAS_ITEM(entity->animation.canvas_item),
			"x", (double) entity->geometry.x1,
			"y", (double) entity->geometry.y1,
			NULL);
	}
}

/* Updates the current pixmap of an item on the canvas */
void update_canvas_animation(Entity *entity) {
	g_assert(entity->animation.pixmaps[entity->animation.frame_no]);
	if(entity->animation.canvas_item) {
		gnome_canvas_item_set(GNOME_CANVAS_ITEM(entity->animation.canvas_item),
				"image", entity->animation.pixmaps[entity->animation.frame_no],
				NULL);
	}
}

/* Tell the gui that the game has ended, and that we should display the title
 * This must be called by everything that calls game.c:end_game, and it must
 * be called before calling game.c:end_game */
void gui_end_game(EndGameStatus status) {
	int pos;
	char *title = NULL;

	gnome_canvas_item_hide(gui->background);
        gnome_canvas_item_show(gui->title_image);
        gnome_canvas_update_now(gui->canvas);

        gtk_widget_set_sensitive(gui->scorelabel, FALSE);
        gtk_widget_set_sensitive(gui->liveslabel, FALSE);
        gtk_widget_set_sensitive(gui->levellabel, FALSE);
	gtk_widget_set_sensitive(gui->menu_pause, FALSE);
	gtk_widget_set_sensitive(gui->menu_end_game, FALSE);

	pos = gnome_score_log((gfloat) gui->game->score, NULL, TRUE);
	switch(status) {
		case ENDGAME_WIN :
			title = _("GNOME Breakout: You win!");
			break;
		case ENDGAME_LOSE :
			title = _("GNOME Breakout: You lose!");
			break;
		case ENDGAME_MENU :
			title = _("GNOME Breakout: Scores");
			break;
		default :
			g_assert_not_reached();
	}

	gnome_scores_display(title, PACKAGE, NULL, pos);
}

/* Tell the gui that the game has begun, and that we should hide the title */
void gui_begin_game(void) {
        gnome_canvas_item_hide(gui->title_image);
        gnome_canvas_item_show(gui->background);
        gnome_canvas_update_now(gui->canvas);

        gtk_widget_set_sensitive(gui->scorelabel, TRUE);
        gtk_widget_set_sensitive(gui->liveslabel, TRUE);
        gtk_widget_set_sensitive(gui->levellabel, TRUE);
	gtk_widget_set_sensitive(gui->menu_pause, TRUE);
	gtk_widget_set_sensitive(gui->menu_end_game, TRUE);
}

gint get_mouse_x_position(void) {
	gint x;
	gint y;
	gtk_widget_get_pointer(GTK_WIDGET(gui->app), &x, &y);
	return x;
}

/*** she's suddenly beautiful         *
 *   we all want something beautiful  *
 *   man, I wish I was beautiful    ***/
