/*
 * Powerup handling
 *
 * Copyright (c) 2000 Michael Pearson <alcaron@ozemail.com.au>
 *
 * This file is licensed under the GNU General Public License. See the file
 * "COPYING" for more details.
 */

#include <stdlib.h>
#include "breakout.h"
#include "gui.h"
#include "anim.h"
#include "bat.h"
#include "collision.h"
#include "powerup.h"

#define POWERUP_SPEED 2

/* Note that it's better to have a low chance of a powerup appearing and
 * powerful powerups, rather than a high chance and weak powerups. This is
 * mainly because powerups cause slowdown :) */
#define POWERUP_CHANCE 20.0

/* Internal functions */
static void powerup_score500(Game *game);
static void powerup_laser(Game *game);
static GList *remove_powerup(GList *powerups, Powerup *powerup);
static void move_powerup(Powerup *powerup);

/* Checks to see whether to create a powerup, and if so, creates one */
void new_powerup(Game *game, int x, int y) {
	Powerup *powerup;

	if((POWERUP_CHANCE * rand()/RAND_MAX) > 1.0)
		return;

	powerup = g_malloc(sizeof(Powerup));
	g_assert(powerup);

	powerup->geometry.x1 = x;
	powerup->geometry.y1 = y;
	powerup->geometry.x2 = x + POWERUP_WIDTH;
	powerup->geometry.y2 = y + POWERUP_HEIGHT;


	powerup->type = (int) (NUMBER_POWERUP_TYPES * rand() / RAND_MAX);
	switch(powerup->type) {
		case POWER_SCORE500 :
			powerup->animation = get_static_animation
					(ANIM_POWERUP_SCORE500);
			break;
		case POWER_LASER :
			powerup->animation = get_static_animation
					(ANIM_POWERUP_LASER);
			break;
		default :
			g_assert_not_reached();
	}

	add_to_canvas((Entity *) powerup);
	game->powerups = g_list_prepend(game->powerups, powerup);
}

/* Adtivates a powerup, and removes it */
void activate_powerup(Game *game, Powerup *powerup) {
	switch(powerup->type) {
		case POWER_SCORE500 :
			powerup_score500(game);
			break;
		case POWER_LASER :
			powerup_laser(game);
			break;
		default :
			g_assert_not_reached();
	}

	game->powerups = remove_powerup(game->powerups, powerup);
}

static void powerup_score500(Game *game) {
	ADD_SCORE(game, 500);
}

static void powerup_laser(Game *game) {
	change_bat_type(game, BAT_LASER);
}

static GList *remove_powerup(GList *powerups, Powerup *powerup) {
        powerups = g_list_remove(powerups, powerup);
        remove_from_canvas((Entity *) powerup);
        g_free(powerup);

        return powerups;
}

/* De-allocate a list of powerups */
void destroy_powerup_list(GList *powerups) {
	while(powerups)
		powerups = remove_powerup(powerups, (Powerup *) powerups->data);
}

/* Make the powerups move, and see if the bat caught them */
void iterate_powerups(Game *game) {
	GList *powerups;
	Powerup *powerup;

	powerups = game->powerups;
	while(powerups) {
		powerup = (Powerup *) powerups->data;
		powerups = g_list_next(powerups);
		move_powerup(powerup);

		if(!bat_powerup_collision(game, powerup))
			if(powerup->geometry.y2 > GAME_HEIGHT)
				game->powerups = remove_powerup(game->powerups, powerup);
		
	}
}

static void move_powerup(Powerup *powerup) {
	powerup->geometry.y1 += POWERUP_SPEED;
	powerup->geometry.y2 += POWERUP_SPEED;
	update_canvas_position((Entity *)powerup);
}
